/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * A machine to gather up arbitrary frames and convert them
 * to raw slinear on demand.
 * 
 * Copyright (C) 2005, Anthony Minessale II.
 *
 * Anthony Minessale <anthmct@yahoo.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 */

#include "asterisk/slinfactory.h"
#include "asterisk/file.h"
#include "asterisk/logger.h"
#include "asterisk/channel.h"
#include "asterisk/pbx.h"
#include "asterisk/module.h"
#include "asterisk/lock.h"
#include "asterisk/cli.h"
#include "asterisk/options.h"
#include "asterisk/app.h"
#include "asterisk/translate.h"


void ast_slinfactory_init(struct ast_slinfactory *sf) 
{
	memset(sf, 0, sizeof(struct ast_slinfactory));
	sf->offset = sf->hold;
	sf->queue = NULL;
}

void ast_slinfactory_destroy(struct ast_slinfactory *sf) 
{
	struct ast_frame *f = NULL;

	if(sf->trans) {
		ast_translator_free_path(sf->trans);
		sf->trans = NULL;
	}

	while((f = sf->queue)) {
		sf->queue = f->next;
		ast_frfree(f);
	}
}

int ast_slinfactory_feed(struct ast_slinfactory *sf, struct ast_frame *f)
{
	int x = -1;
	struct ast_frame *frame = NULL, *frame_ptr = NULL;

	if (f) {
		x = 0;
		if (f->subclass != AST_FORMAT_SLINEAR) {
			if (sf->trans && f->subclass != sf->format) {
				ast_translator_free_path(sf->trans);
			}
			if (!sf->trans) {
				if ((sf->trans = ast_translator_build_path(AST_FORMAT_SLINEAR, f->subclass)) == NULL) {
					ast_log(LOG_WARNING, "Cannot build a path from %s to slin\n", ast_getformatname(sf->format));
				} else {
					sf->format = f->subclass;
				}
			}
		}
	
		if(sf->trans) {
			frame = ast_translate(sf->trans, f, 0);
		} else {
			frame = ast_frdup(f);
		}

		if (frame) {
			for (frame_ptr = sf->queue; frame_ptr && frame_ptr->next; frame_ptr=frame_ptr->next) {
				x++;
			}
			if (frame_ptr) {
				frame_ptr->next = frame;
			} else {
				sf->queue = frame;
			}
			frame->next = NULL;
			sf->size += frame->datalen;	
		}


	}

	return x;
}

int ast_slinfactory_read(struct ast_slinfactory *sf, short *buf, size_t bytes) {
	struct ast_frame *frame_ptr = NULL;
	int sofar = 0, ineed, remain;
	void *frame_data, *offset = buf;

	while (sofar < bytes) {
		ineed = bytes - sofar;

		if(sf->holdlen) {
			if ((sofar + sf->holdlen) <= ineed) {
				memcpy(offset, sf->hold, sf->holdlen);
				sofar += sf->holdlen;
				offset += sf->holdlen;
				sf->holdlen = 0;
				sf->offset = sf->hold;
			} else {
				remain = sf->holdlen - ineed;
				memcpy(offset, sf->offset, ineed);
				sofar += ineed;
				sf->offset += ineed;
				sf->holdlen = remain;
			}
			continue;
		}
		
		if ((frame_ptr = sf->queue)) {
			sf->queue = frame_ptr->next;
			frame_data = frame_ptr->data;
			
			if ((sofar + frame_ptr->datalen) <= ineed) {
				memcpy(offset, frame_data, frame_ptr->datalen);
				sofar += frame_ptr->datalen;
				offset += frame_ptr->datalen;
			} else {
				remain = frame_ptr->datalen - ineed;
				memcpy(offset, frame_data, ineed);
				sofar += ineed;
				frame_data += ineed;
				memcpy(sf->hold, frame_data, remain);
				sf->holdlen = remain;
			}
			ast_frfree(frame_ptr);
		} else {
			break;
		}
	}

	sf->size -= sofar;
	return sofar;
}




